@@ -1,21 +1,25 @@
-glmm_final_smooth_noRE<-function(y,X,Phi,penal.vec,Delta_start,steps=1000,family,overdispersion,
+glmm_final_smooth_noRE<-function(y,X,Phi,penal.vec,K,Delta_start,steps=1000,family,overdispersion,
                             phi,nue=1,print.iter.final=FALSE,eps.final=1e-5)
 {
 dim.smooth<-dim(Phi)[2]  
 N<-length(y)
 lin<-ncol(as.matrix(X))
 Eta<-cbind(X,Phi)%*%Delta_start
-Mu<-as.vector(family$linkinv(Eta))
-Sigma<-as.vector(family$variance(Mu))
-if(overdispersion)
-  Sigma<-Sigma*phi
-D<-as.vector(family$mu.eta(Eta))
-#W0_inv<-D*1/Sigma*D
+
+if(is.null(family$multivariate)){
+  D<-family$mu.eta(Eta)
+  Mu<-family$linkinv(Eta)
+  SigmaInv <- 1/family$variance(Mu)
+}else{
+  Eta_cat <- matrix(Eta, byrow = TRUE, ncol = K)
+  Mu_cat <- family$linkinv(Eta_cat)
+  D <- family$deriv.mat(Mu_cat)
+  SigmaInv <- family$SigmaInv(Mu_cat)
+  Mu <- c(t(Mu_cat))
+}
 
 if(print.iter.final)
   message("Final Re-estimation Iteration ", 1)
-#print(paste("Final Re-estimation Iteration ", 1,sep=""))
-
 
 Z_alles<-cbind(X,Phi)
 
@@ -26,15 +30,18 @@ Delta<-matrix(0,steps,(lin+dim.smooth))
 Eta.ma<-matrix(0,steps+1,N)
 Eta.ma[1,]<-Eta
 
-
 l=1
 opt<-steps
 
 
-score_vec<-t(Z_alles)%*%((y-Mu)*D*1/Sigma)-P1%*%Delta[1,]
-F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
-
-
+if(is.null(family$multivariate)){
+  D <- drop(D);SigmaInv <- drop(SigmaInv)
+  score_vec <- t(Z_alles)%*%((y-Mu)*D*SigmaInv)-P1%*%Delta[1,]
+  F_gross<-t(Z_alles)%*%(Z_alles*D*SigmaInv*D)+P1
+}else{
+  score_vec <- t(Z_alles)%*%(D%*%(SigmaInv%*%(y-Mu)))-P1%*%Delta[1,]
+  F_gross<-t(Z_alles)%*%(D%*%(SigmaInv%*%(t(D)%*%Z_alles)))+P1
+}
 
 InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
 if(class(InvFisher)=="try-error")
@@ -42,8 +49,6 @@ InvFisher<-try(solve(F_gross),silent=TRUE)
 if(class(InvFisher)=="try-error")
 stop("Fisher matrix not invertible")
 
-
-
 half.index<-0
 solve.test<-FALSE
 Delta_r<-InvFisher%*%score_vec
@@ -51,43 +56,33 @@ Delta_r<-InvFisher%*%score_vec
 ######### big while loop for testing if the update leads to Fisher matrix which can be inverted
 while(!solve.test)
 {  
+  if(half.index>50)
+    half.index<-Inf
   
-solve.test2<-FALSE  
-while(!solve.test2)
-{  
 Delta[1,]<-Delta_start+nue*(0.5^half.index)*Delta_r
 
 Eta<-Z_alles%*%Delta[1,]
 
-Mu<-as.vector(family$linkinv(Eta))
-Sigma<-as.vector(family$variance(Mu))
-D<-as.vector(family$mu.eta(Eta))
-
-
-if (overdispersion)
-{  
-  F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
-  InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
-  if(class(InvFisher)=="try-error")
-    InvFisher<-try(solve(F_gross),silent=TRUE)  
-  if(class(InvFisher)=="try-error")
-  {
-    half.index<-half.index+1  
-  }else{
-    solve.test2<-TRUE 
-}}else{
-    solve.test2<-TRUE
-}}
-
-if(overdispersion)
-{
-FinalHat<-(Z_alles*sqrt(Sigma*D*1/Sigma*D))%*%InvFisher%*%t(Z_alles*sqrt(D*1/Sigma*D*1/Sigma))#E-Uu
-phi<-(sum((y-Mu)^2/Mu))/(N-sum(diag(FinalHat)))
-Sigma<-Sigma*phi
+if(is.null(family$multivariate)){
+  D<-family$mu.eta(Eta)
+  Mu<-family$linkinv(Eta)
+  SigmaInv <- 1/family$variance(Mu)
+}else{
+  Eta_cat <- matrix(Eta, byrow = TRUE, ncol = K)
+  Mu_cat <- family$linkinv(Eta_cat)
+  D <- family$deriv.mat(Mu_cat)
+  SigmaInv <- family$SigmaInv(Mu_cat)
+  Mu <- c(t(Mu_cat))
 }
 
-score_vec<-t(Z_alles)%*%((y-Mu)*D*1/Sigma)-P1%*%Delta[1,]
-F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
+if(is.null(family$multivariate)){
+  D <- as.vector(D);SigmaInv <- as.vector(SigmaInv)
+  score_vec <- t(Z_alles)%*%((y-Mu)*D*SigmaInv)-P1%*%Delta[1,]
+  F_gross<-t(Z_alles)%*%(Z_alles*D*SigmaInv*D)+P1
+}else{
+  score_vec <- t(Z_alles)%*%(D%*%(SigmaInv%*%(y-Mu)))-P1%*%Delta[1,]
+  F_gross<-t(Z_alles)%*%(D%*%(SigmaInv%*%(t(D)%*%Z_alles)))+P1
+}
 
 InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
 if(class(InvFisher)=="try-error")
@@ -102,6 +97,9 @@ if(class(InvFisher)=="try-error")
 
 Eta.ma[2,]<-Eta
 
+y_dach<-as.vector(family$linkinv(Eta))
+Dev_neu<-sum(family$dev.resids(y,y_dach,wt=rep(1,N))^2)
+
 ###############################################################################################################################################
 ################################################################### Main Iteration ###################################################################
 eps<-eps.final*sqrt(length(Delta_r))
@@ -122,50 +120,33 @@ first.time<-FALSE
 while(!solve.test)
 {  
   
-solve.test2<-FALSE  
-while(!solve.test2)
-{  
- if(half.index>100)
- half.index<-Inf
-
+  if(half.index>50)
+    half.index<-Inf
+  
 Delta[l,]<-Delta[l-1,]+nue*(0.5^half.index)*Delta_r
 
 Eta<-Z_alles%*%Delta[l,]
-Mu<-as.vector(family$linkinv(Eta))
-Sigma<-as.vector(family$variance(Mu))
-D<-as.vector(family$mu.eta(Eta))
 
-if (overdispersion)
-{  
-  F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
-  InvFisher2<-try(chol2inv(chol(F_gross)),silent=TRUE)
-  if(class(InvFisher2)=="try-error")
-    InvFisher2<-try(solve(F_gross),silent=TRUE)  
-  if(class(InvFisher2)=="try-error")
-  {
-    half.index<-half.index+1  
-  }else{
-    if(!first.time)
-    half.index.final<-half.index
-    solve.test2<-TRUE 
-    first.time<-TRUE
-  }}else{
-    if(!first.time)
-    half.index.final<-half.index
-    solve.test2<-TRUE 
-    first.time<-TRUE
-  }}
-
-
-if(overdispersion)
-{
-FinalHat<-(Z_alles*sqrt(Sigma*D*1/Sigma*D))%*%InvFisher%*%t(Z_alles*sqrt(D*1/Sigma*D*1/Sigma))#E-Uu
-phi<-(sum((y-Mu)^2/Mu))/(N-sum(diag(FinalHat)))
-Sigma<-Sigma*phi
+if(is.null(family$multivariate)){
+  D<-family$mu.eta(Eta)
+  Mu<-family$linkinv(Eta)
+  SigmaInv <- 1/family$variance(Mu)
+}else{
+  Eta_cat <- matrix(Eta, byrow = TRUE, ncol = K)
+  Mu_cat <- family$linkinv(Eta_cat)
+  D <- family$deriv.mat(Mu_cat)
+  SigmaInv <- family$SigmaInv(Mu_cat)
+  Mu <- c(t(Mu_cat))
 }
 
-score_vec<-t(Z_alles)%*%((y-Mu)*D*1/Sigma)-P1%*%Delta[l,]
-F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
+if(is.null(family$multivariate)){
+  D <- as.vector(D);SigmaInv <- as.vector(SigmaInv)
+  score_vec <- t(Z_alles)%*%((y-Mu)*D*SigmaInv)-P1%*%Delta[l,]
+  F_gross<-t(Z_alles)%*%(Z_alles*D*SigmaInv*D)+P1
+}else{
+  score_vec <- t(Z_alles)%*%(D%*%(SigmaInv%*%(y-Mu)))-P1%*%Delta[l,]
+  F_gross<-t(Z_alles)%*%(D%*%(SigmaInv%*%(t(D)%*%Z_alles)))+P1
+}
 
 InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
 if(class(InvFisher)=="try-error")
@@ -191,47 +172,30 @@ if(finish ||  finish2)
 
 opt<-l
 
-if(!solve.test)
-{
-  Delta[l,]<-Delta[l-1,]+nue*(0.5^half.index.final)*Delta_r
-  Eta<-Z_alles%*%Delta[l,]
-  Mu<-as.vector(family$linkinv(Eta))
-  Sigma<-as.vector(family$variance(Mu))
-  D<-as.vector(family$mu.eta(Eta))
-  
-  if (overdispersion)
-  {  
-    F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
-    InvFisher2<-try(chol2inv(chol(F_gross)),silent=TRUE)
-    if(class(InvFisher2)=="try-error")
-      InvFisher2<-try(solve(F_gross),silent=TRUE)  
-  }
-
-  FinalHat<-(Z_alles*sqrt(Sigma*D*1/Sigma*D))%*%InvFisher%*%t(Z_alles*sqrt(D*1/Sigma*D*1/Sigma))
-  complexity<-sum(diag(FinalHat))
-
-if(overdispersion)
-{
-  phi<-(sum((y-Mu)^2/Mu))/(N-complexity)
-  Sigma<-Sigma*phi
-}
+if(is.null(family$multivariate)){
+  W_opt <- D*SigmaInv*D
+  FinalHat<-(Z_alles*sqrt(W_opt))%*%InvFisher%*%t(Z_alles*sqrt(W_opt))
+}else{
+  W_opt <- D%*%(SigmaInv%*%t(D))
+  W_inv_t <- chol(W_opt)
+  FinalHat<-W_inv_t%*%(Z_alles%*%(InvFisher%*%(t(Z_alles)%*%t(W_inv_t))))
 }
 
-FinalHat<-(Z_alles*sqrt(Sigma*D*1/Sigma*D))%*%InvFisher%*%t(Z_alles*sqrt(D*1/Sigma*D*1/Sigma))
 complexity<-sum(diag(FinalHat))
 
-Deltafinal<-Delta[l,]
+if(overdispersion)
+  phi<-(sum((y-Mu)^2/family$variance(Mu)))/(N-complexity)
 
+
+Deltafinal<-Delta[l,]
 Standard_errors<-sqrt(diag(InvFisher))
 
-
 #### compare Wood's Book on page 167
-  P1a<-c(rep(0,lin+dim.smooth))
-  P1a<-diag(P1a)
-  
-#### compare Wood's Book on page 167
-EDF.matrix<-InvFisher%*%(t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1a)
-
+  if(is.null(family$multivariate)){
+    EDF.matrix<-InvFisher%*%(t(Z_alles)%*%(Z_alles*D*SigmaInv*D))
+  }else{
+    EDF.matrix<-InvFisher%*%(t(Z_alles)%*%(D%*%(SigmaInv%*%(t(D)%*%Z_alles))))
+  }
 ret.obj=list()
 ret.obj$opt<-opt
 ret.obj$Delta<-Deltafinal
